import React, {useEffect, useMemo, useState} from 'react';
import {Input, Menu, Tree} from 'antd';
import {FolderOutlined, FileMarkdownOutlined, FileOutlined} from '@ant-design/icons';
import "./index.less"
import {getFileNameByPath} from "../../utils/PathOperate";
const {Search} = Input;
import {useSelector, useDispatch} from "react-redux";
import {addExpandedKeys, addTableBarItem, setExpandedKeys} from "../../redux/tableBarItem_reducer";
import {readDir} from "../../utils/File";
import {nextDirAdd,refreshDir as refreshDirReducer} from "../../redux/dirMessage_reducer";
import {isEmpty} from "../../utils/ObjectUtils";
import UpdateFileName from "./UpdateFileName";
import RefreshDir from "./RefreshDir";
import CloseDir from "./CloseDir";
import DirAddFile from "./DirAddFile";
import DirDeleteFile from "./DirDeleteFile";
import DirAddDir from "./DirAddDir";
// const defaultData = [];
// 将树平铺用于查找
const dataList = [];
const generateList = (data) => {
    for (let i = 0; i < data.length; i++) {
        const node = data[i];
        const {key, title, icon} = node;
        dataList.push({
            key,
            title,
            icon
        });
        if (node.children) {
            generateList(node.children);
        }
    }
};
// generateList(defaultData);
const getParentKey = (key, tree) => {
    let parentKey;
    for (let i = 0; i < tree.length; i++) {
        const node = tree[i];
        if (node.children) {
            if (node.children.some((item) => item.key === key)) {
                parentKey = node.key;
            } else if (getParentKey(key, node.children)) {
                parentKey = getParentKey(key, node.children);
            }
        }
    }
    return parentKey;
};

function generateChildList(fileList) {
    const result = []
    for (let i = 0; i < fileList.length; i++) {
        const {fileName, filePath, dirFlag, children} = fileList[i];
        const childrenM = []
        if (Array.isArray(children) && children.length > 0) {
            childrenM.push(...generateChildList(children));
        }
        result.push({
            "key": filePath,
            "title": titleExtended(fileName, filePath),
            "icon": dirFlag ? <FolderOutlined/> : fileName.endsWith(".md") ? <FileMarkdownOutlined/> : <FileOutlined/>,
            "dirFlag": dirFlag,
            "children": childrenM
        });
    }
    return result;
}

const titleExtended = (fileName, filePath) => {
    return <span title={filePath}>{fileName}</span>
}
/**
 * 将文件信息改为树信息
 * @param fileDirDate
 * @returns {*[]}
 */
const flushTree = (fileDirDate) => {
    const defaultValueStateSet = [];
    if (Array.isArray(fileDirDate) && fileDirDate.length > 0) {
        for (let i = 0; i < fileDirDate.length; i++) {
            const node = fileDirDate[i];
            console.log("node:", node)
            const {fileName, filePath, children, dirFlag} = node;
            const childrenM = []
            if (Array.isArray(children) && children.length > 0) {
                childrenM.push(...generateChildList(children));
            }
            defaultValueStateSet.push({
                "key": filePath,
                // 修改属性后此处也需要修改
                "title": titleExtended(getFileNameByPath(fileName), filePath),
                "icon": <FolderOutlined/>,
                "dirFlag": dirFlag,
                "children": childrenM
            });

        }
    }
    return defaultValueStateSet;
}
const ItemTree = (prop) => {
    console.log("prop.filePath:", prop.filePath)
    const dispatch = useDispatch()
    // const [expandedKeys, setExpandedKeys] = useState([]);
    const expandedKeys = useSelector(state => state.tableBarItem.expandedKeyList)
    const [searchValue, setSearchValue] = useState('');
    const [autoExpandParent, setAutoExpandParent] = useState(true);
    // let filePath = useSelector(state => state.dirMessage.data);
    const [defaultValueState, setDefaultValueState] = useState(flushTree(prop.filePath));
    const [state, setState] = useState();
    useEffect(() => {
        setDefaultValueState(flushTree(prop.filePath))
    }, [prop]);
    const onExpand = (newExpandedKeys) => {
        dispatch(setExpandedKeys(newExpandedKeys));
        setAutoExpandParent(false);
    };

    const addChildNode = (valueState, fileStateList) => {
        valueState.forEach(file => {
            if (fileStateList[0].key.startsWith(file.key)) {
                if (file.key === fileStateList[0].key) {
                    file.children = fileStateList[0].children
                } else if (Array.isArray(file.children) && file.children.length > 0) {
                    addChildNode(file.children, fileStateList)
                }
            }
        })
    }
    const itemTreeAddFile = (dirMessage)=>{
        setDefaultValueState(flushTree(dirMessage))
        // return
        // let newFile = {
        //     "fileName": getFileFullNameByPath(filePath),
        //     "filePath": filePath,
        //     "dirFlag": false,
        //     "children": []
        // }
        // // 查找对应的目录并添加信息
        // let fileDir = getFileDirByPath(filePath)
        // let updateValueState = [...prop.filePath]
        // updateValueState.forEach(file=>{
        //     if (fileDir===file.filePath && file.dirFlag){
        //         file.children=[...file.children,newFile]
        //     } else if (fileDir.startsWith(file.filePath) && file.dirFlag ){
        //         treeItemChildAddFile(file.children,newFile,fileDir)
        //     }
        // })
        // setDefaultValueState(flushTree(updateValueState))
    }
    function treeItemChildAddFile(treeItemList,newFile,fileDir){
        treeItemList.forEach(file=>{
            if (fileDir===file.filePath && file.dirFlag){
                file.children=[...file.children,newFile]
            } else if (fileDir.startsWith(file.filePath) && file.dirFlag){
                treeItemChildAddFile(file.children,newFile,fileDir)
            }
        })
    }

    const onChange = (e) => {
        const {value} = e.target;
        const newExpandedKeys = dataList
            .map((item) => {
                if (item.title.indexOf(value) > -1) {
                    return getParentKey(item.key, defaultValueState);
                }
                return null;
            })
            .filter((item, i, self) => !!(item && self.indexOf(item) === i));
        dispatch(setExpandedKeys(newExpandedKeys));
        setSearchValue(value);
        setAutoExpandParent(true);
    };

    const refreshDir = (filePath) => {
        readDir(filePath).then(fileStateList => {
            if (Array.isArray(fileStateList[0].children) && fileStateList[0].children.length > 0) {
                dispatch(refreshDirReducer({selectDirKey: filePath, fileStateList}))
                // 添加下级节点
                addChildNode(defaultValueState, flushTree(fileStateList))
                const result = [...defaultValueState]
                console.log("[...defaultValueState]:", result)
                setDefaultValueState(result)
                // 打开当前目录
                dispatch(addExpandedKeys([filePath]));
                setAutoExpandParent(false);
            }
        })
    }
    const onSelect = (selectedKeys, e) => {
        if (e.selected) {
            console.log('onSelect.selectedKeys', selectedKeys, e)
            if (e.node.dirFlag) {
                // 加载目录下一级文件信息
                if (!Array.isArray(e.node.children) || e.node.children.length === 0) {
                    readDir(e.node.key).then(fileStateList => {
                        if (Array.isArray(fileStateList[0].children) && fileStateList[0].children.length > 0) {
                            dispatch(nextDirAdd({selectDirKey: e.node.key, fileStateList}))
                            // 添加下级节点
                            addChildNode(defaultValueState, flushTree(fileStateList))
                            const result = [...defaultValueState]
                            console.log("[...defaultValueState]:", result)
                            setDefaultValueState(result)
                            // 打开当前目录
                            dispatch(addExpandedKeys([e.node.key]));
                            setAutoExpandParent(false);
                        }
                    })
                } else {
                    // 打开当前目录
                    dispatch(addExpandedKeys([e.node.key]));
                    setAutoExpandParent(false);
                }
            } else {
                // 打开文件
                dispatch(addTableBarItem({
                    label: e.node.title.props.children,
                    children: e.node.key,
                    key: e.node.key,
                    activeKey: e.node.key
                }))
            }
        }
    }
    // const treeData = useMemo(() => {
    //     const loop = (data) =>
    //         data.map((item) => {
    //             const strTitle = item.title;
    //             const index = strTitle.indexOf(searchValue);
    //             const beforeStr = strTitle.substring(0, index);
    //             const afterStr = strTitle.slice(index + searchValue.length);
    //             const title =
    //                 index > -1 ? (
    //                     <span>{beforeStr}
    //                         <span className="site-tree-search-value">{searchValue}</span>{afterStr}</span>
    //                 ) : (
    //                     <span>{strTitle}</span>
    //                 );
    //             if (item.children) {
    //                 return {
    //                     title,
    //                     key: item.key,
    //                     icon: item.icon,
    //                     children: loop(item.children),
    //                 };
    //             }
    //             return {
    //                 title,
    //                 icon: item.icon,
    //                 key: item.key,
    //             };
    //         });
    //     return loop(defaultData);
    // }, [searchValue]);
    const treeNodeOnRightClick = (e) => {
        console.log("e,node", e)
        setState({
            rightClickNodeTreeItem: {
                pageX: e.event.pageX,
                pageY: e.event.pageY,
                key: e.node.key,
                dirFlag: e.node.dirFlag,
                title: getFileNameByPath(e.node.key),
            }
        });
    }
    const getNodeTreeRightClickMenu = () => {
        console.log("state", state, isEmpty(state))
        if (isEmpty(state)) {
            return
        }
        const {pageX, pageY, title, dirFlag, key} = {...state.rightClickNodeTreeItem};
        const tmpStyle = {
            position: 'absolute',
            left: `${pageX}px`,
            top: `${pageY}px`,
            popupBg: "#ec0f0f",
            zIndex: "9999"
        };
        const menuItem = [
            !dirFlag &&
            <Menu.Item key='1'>
                <UpdateFileName fileName={title} filePath={key} closeMenu={() => setState("")}/>
            </Menu.Item>,
            dirFlag &&
            <Menu.Item key='2'>
                <RefreshDir filePath={key} refreshDir={refreshDir} closeMenu={() => setState("")}/>
            </Menu.Item>,
            dirFlag &&
            <Menu.Item key='7'>
                <DirAddDir fileDir={key} closeMenu={() => setState("")}/>
            </Menu.Item>,
            dirFlag &&
            <Menu.Item key='3'>
                <DirAddFile fileDir={key} itemTreeAddFile = {(dirMessage)=>itemTreeAddFile(dirMessage)} closeMenu={() => setState("")}/>
            </Menu.Item>,
            !dirFlag &&
            <Menu.Item key='4'>
                <DirDeleteFile filePath={key} closeMenu={() => setState("")}></DirDeleteFile>
            </Menu.Item>,
            dirFlag &&
            <Menu.Item key='5'>
                <CloseDir filePath={key} closeMenu={() => setState("")}/>
            </Menu.Item>,
            <Menu.Item key='6' onClick={() => {setState("")}}>关闭菜单</Menu.Item>
        ]
        return (
            <div>
                <Menu style={tmpStyle}>
                    {menuItem}
                </Menu>
            </div>
        )
    }

    return (
        <div style={{height: "100%"}}>
            {getNodeTreeRightClickMenu()}
            <Search
                style={{marginBottom: 8,}}
                placeholder="Search"
                onChange={onChange}
            />
            <Tree
                // 展开/收起节点时触发
                onExpand={onExpand}
                //（受控）展开指定的树节点
                expandedKeys={expandedKeys}
                // 是否自动展开父节点
                autoExpandParent={autoExpandParent}
                showIcon={true}
                // selectedKeys={[useSelector(state => state.tableBarItem.activeKey)]}
                defaultExpandedKeys={useSelector(state => state.tableBarItem.expandedKeyList)}
                // treeNodes 数据，如果设置则不需要手动构造 TreeNode 节点（key 在整个树范围内唯一）
                treeData={defaultValueState}
                onSelect={onSelect}
                onRightClick={treeNodeOnRightClick}
            />
        </div>
    );
};
export default ItemTree;